///////////////////////////////////////////////////////////////////////////////////////
// Copyright (C) 2006-2015 Esper Team. All rights reserved.                           /
// http://esper.codehaus.org                                                          /
// ---------------------------------------------------------------------------------- /
// The software in this package is published under the terms of the GPL license       /
// a copy of which has been included with this distribution in the license.txt file.  /
///////////////////////////////////////////////////////////////////////////////////////

using System;

using com.espertech.esper.compat;
using com.espertech.esper.compat.collections;

namespace com.espertech.esper.client
{
    /// <summary>
    /// Parameter holder for parameterized on-demand queries that are prepared with substitution parameters and that
    /// can be executed efficiently multiple times with different actual values for parameters.
    /// <para />A pre-compiled query can only be executed when actual values for all
    /// substitution parameters are set.
    /// </summary>
    public interface EPOnDemandPreparedQueryParameterized
    {
        /// <summary>
        /// Sets the value of the designated parameter using the given object.
        /// </summary>
        /// <param name="parameterIndex">the first parameter is 1, the second is 2, ...</param>
        /// <param name="value">the object containing the input parameter value</param>
        /// <throws>EPException if the substitution parameter could not be located</throws>
        void SetObject(int parameterIndex, object value) ;
    }
    
}
